/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Option template for option modules.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.cpp file.
 */
#include "options/ff_options.h"

#include <iostream>

#include "base/check.h"
#include "options/option_exception.h"

namespace cvc5::internal::options {

// clang-format off
std::ostream& operator<<(std::ostream& os, FfSolver mode)
{
  switch(mode)
  {
    case FfSolver::GB: return os << "gb";
    case FfSolver::SPLIT_GB: return os << "split";
    default: Unreachable();
  }
  return os;
}
FfSolver stringToFfSolver(const std::string& optarg)
{
  if (optarg == "gb") return FfSolver::GB;
  else if (optarg == "split") return FfSolver::SPLIT_GB;
  else if (optarg == "help")
  {
    std::cerr << R"FOOBAR(
  Which field solver
Available modes for --ff-solver are:
+ gb (default)
  use a groebner basis for the whole system
+ split
  use multiple groebner bases for partitions of the system
)FOOBAR";
    std::exit(1);
  }
  throw OptionException(std::string("unknown option for --ff-solver: `") +
                        optarg + "'.  Try --ff-solver=help.");
}
// clang-format on

}  // namespace cvc5::internal::options
