/******************************************************************************
 * Top contributors (to current version):
 *   Gereon Kremer, Aina Niemetz, Morgan Deters
 *
 * This file is part of the cvc5 project.
 *
 * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
 * in the top-level source directory and their institutional affiliations.
 * All rights reserved.  See the file COPYING in the top-level source
 * directory for licensing information.
 * ****************************************************************************
 *
 * Contains code for handling command-line options.
 *
 * For each <module>_options.toml configuration file, mkoptions.py
 * expands this template and generates a <module>_options.h file.
 */

#include "cvc5_private.h"

#ifndef CVC5__OPTIONS__FF_H
#define CVC5__OPTIONS__FF_H

#include "options/options.h"

// clang-format off

// clang-format on

namespace cvc5::internal::options {

// clang-format off
enum class FfSolver
{
  GB, SPLIT_GB,
  __MAX_VALUE = SPLIT_GB
};
std::ostream& operator<<(std::ostream& os, FfSolver mode);
FfSolver stringToFfSolver(const std::string& optarg);

// clang-format on

#if defined(CVC5_MUZZLED) || defined(CVC5_COMPETITION_MODE)
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT false
#else /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */
#  define DO_SEMANTIC_CHECKS_BY_DEFAULT true
#endif /* CVC5_MUZZLED || CVC5_COMPETITION_MODE */

struct HolderFF
{
// clang-format off
  bool ffBitsum = false;
  bool ffBitsumWasSetByUser = false;
  bool ffDisjunctiveBit = false;
  bool ffDisjunctiveBitWasSetByUser = false;
  bool ffFieldPolys = false;
  bool ffFieldPolysWasSetByUser = false;
  FfSolver ffSolver = FfSolver::GB;
  bool ffSolverWasSetByUser = false;
  bool ffTraceGb = true;
  bool ffTraceGbWasSetByUser = false;
// clang-format on
};

#undef DO_SEMANTIC_CHECKS_BY_DEFAULT


}  // namespace cvc5::internal::options

#endif /* CVC5__OPTIONS__FF_H */
